/*****************************************************************************************
 *                                                                                       *
 * OpenSpace                                                                             *
 *                                                                                       *
 * Copyright (c) 2014-2017                                                               *
 *                                                                                       *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this  *
 * software and associated documentation files (the "Software"), to deal in the Software *
 * without restriction, including without limitation the rights to use, copy, modify,    *
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and to    *
 * permit persons to whom the Software is furnished to do so, subject to the following   *
 * conditions:                                                                           *
 *                                                                                       *
 * The above copyright notice and this permission notice shall be included in all copies *
 * or substantial portions of the Software.                                              *
 *                                                                                       *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,   *
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A         *
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT    *
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF  *
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE  *
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                         *
 ****************************************************************************************/

#include <openspace/properties/stringproperty.h>

#include <ghoul/lua/ghoul_lua.h>

namespace openspace {
namespace properties {

REGISTER_TEMPLATEPROPERTY_SOURCE(StringProperty, std::string, "",
[](lua_State* state, bool& success) -> std::string {
        success = lua_isstring(state, -1) == 1;
        if (success) {
            return lua_tostring(state, -1);
        }
        else {
            return "";
        }
    },
[](lua_State* state, std::string value) -> bool {
        lua_pushstring(state, value.c_str());
        return true;
    },
[](std::string value, bool& success) -> std::string {
    // An incoming string is of the form
    // "value"
    // so we want to remove the leading and trailing " characters
    if (value.size() > 2 && (value[0] == '"' && value[value.size() - 1] == '"')) {
        // Removing the first and last "
        success = true;
        return value.substr(1, value.size() - 2);
    }
    success = false;
    return value;
},
[](std::string& outValue, std::string inValue) -> bool {
    outValue = "\"" + inValue + "\"";
    return true;
},
LUA_TSTRING
);

} // namespace properties
} // namespace openspace
